@@ -1,5 +1,5 @@
 from template_ns4 import *
 
-d = make_Class_DD();
+d = make_Class_DD()
 if d.test() != "test":
-	raise RuntimeError
+    raise RuntimeError
